/*******************************************************************
$Header: C:/Symbol/Csp32Proj/VcCspDemo/src/vcs/CommDialog.cpv  1.0  15 Mar 1999  12:00:00 jTykowski $
 *
 *  Copyright 1999 Peninsula Solutions, Inc., all rights reserved.
 *  Copyright 1999 Symbol Technologies, Inc., all rights reserved.
 *
 *          ***************************************
 *          * This file was generated by:         *
 *          *       Peninsula Solutions, Inc.     *
 *          *       1265 S. Semoran Blvd.         *
 *          *       Suite 1247                    *     
 *          *       Winter Park, FL 32792         *
 *          *       Phone: (407) 673-6544         *
 *          *       Fax:   (407) 673-6545         *
 *          * E-mail: info@PeninsulaSolutions.com *
 *          *                                     *
 *          * under a grant from:                 *
 *          *       Symbol Technologies, Inc.     *
 *          *       Holtsville, NY                *
 *          ***************************************
 *
 * The user is granted a license to use and modify this file for their own
 * purposes so long as the Copyright information remains intact.
 *
 *    MODULE NAME:   CommDialog.cpp
 *
 *    DESCRIPTION:
 *                   This file provides a comm port selection
 *                   dialog for the user. It provides an example
 *                   of how the programmer can determine which comm ports
 *                   are suitable for use with a CSP device and
 *                   present the list of valid options to the user.
 *
 *    AUTHOR:        Joe Tykowski
 *
 *    DATE CREATED:  03/15/99
 *
 *    HISTORY:
 *
 *       03/15/99    Initial Development
 *
$Log: C:/Symbol/Csp32Proj/VcCspDemo/src/vcs/CommDialog.cpv $
 *******************************************************************/

/** include files **/
#include "stdafx.h"
#include "vccspdemo.h"
#include "CommDialog.h"
#include "csp32.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

long    nActivePortSave;

BOOL    bSerialPort1;
BOOL    bSerialPort2;
BOOL    bSerialPort3;
BOOL    bSerialPort4;

/////////////////////////////////////////////////////////////////////////////
// CCommDialog dialog


CCommDialog::CCommDialog(CWnd* pParent /*=NULL*/)
    : CDialog(CCommDialog::IDD, pParent)
{
    //{{AFX_DATA_INIT(CCommDialog)
    m_nActivePort = 0;
    //}}AFX_DATA_INIT
}


void CCommDialog::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CCommDialog)
    DDX_Text(pDX, IDC_COMM_SELECT, m_nActivePort);
    DDV_MinMaxInt(pDX, m_nActivePort, -1, 3);
    //}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CCommDialog, CDialog)
    //{{AFX_MSG_MAP(CCommDialog)
    ON_BN_CLICKED(IDC_COM1, OnCom1)
    ON_BN_CLICKED(IDC_COM2, OnCom2)
    ON_BN_CLICKED(IDC_COM3, OnCom3)
    ON_BN_CLICKED(IDC_COM4, OnCom4)
    ON_BN_CLICKED(IDC_COMMTEST, OnCommtest)
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCommDialog message handlers
/*
 ** BOOL CCommDialog::OnInitDialog() 
 *
 *  FILENAME: C:\Symbol\Csp32Proj\VcCspDemo\Src\CommDialog.cpp
 *
 *  PARAMETERS:     void
 *
 *  DESCRIPTION:    This function checks each of the comm ports
 *                  initially to determine if it is suitable for
 *                  use with the CSP device. Unsuitable ports are
 *                  made indeterminate/gray. Suitable ports are either
 *                  selected or selectable.
 *
 *  RETURNS:        TRUE
 *
 */

BOOL CCommDialog::OnInitDialog() 
{
    CDialog::OnInitDialog();
    
    // Get a pointer to each of the check boxes representing the comm ports...
    CButton *Com1 = (CButton *) GetDlgItem(IDC_COM1);
    CButton *Com2 = (CButton *) GetDlgItem(IDC_COM2);
    CButton *Com3 = (CButton *) GetDlgItem(IDC_COM3);
    CButton *Com4 = (CButton *) GetDlgItem(IDC_COM4);

    // retrieve the currently selected port setting...
    nActivePortSave = m_nActivePort;

    // Has COM1 already been selected?
    if (nActivePortSave == COM1)
    {
        // Yes, show it as selected by default...
        bSerialPort1 = TRUE;
        Com1->SetCheck(1);  
    }
    // is COM1 suitable for CSP devices?
    else if (cspGetCommInfo(COM1) == SERIAL_RS232)
    {
        // Yes, it is suitable
        bSerialPort1 = TRUE;
        if (nActivePortSave == COM1)
            Com1->SetCheck(1);  
        else
            Com1->SetCheck(0);  
    }
    else
    {
        // No, it is not suitable, gray the control...
        bSerialPort1 = FALSE;
        Com1->SetCheck(2);  
    }

    // Has COM2 already been selected?
    if (nActivePortSave == COM2)
    {
        bSerialPort2 = TRUE;
        Com2->SetCheck(1);  
    }
    // is COM2 suitable for CSP devices?
    else if (cspGetCommInfo(COM2) == SERIAL_RS232)
    {
        // Yes, it is suitable
        bSerialPort2 = TRUE;
        if (nActivePortSave == COM2)
            Com2->SetCheck(1);  
        else
            Com2->SetCheck(0);  
    }
    else
    {
        // No, it is not suitable, gray the control...
        bSerialPort2 = FALSE;
        Com2->SetCheck(2);  
    }

    // Has COM3 already been selected?
    if (nActivePortSave == COM3)
    {
        bSerialPort3 = TRUE;
        Com3->SetCheck(1);  
    }
    // is COM3 suitable for CSP devices?
    else if (cspGetCommInfo(COM3) == SERIAL_RS232)
    {
        // Yes, it is suitable
        bSerialPort3 = TRUE;
        if (nActivePortSave == COM3)
            Com3->SetCheck(1);  
        else
            Com3->SetCheck(0);  
    }
    else
    {
        // No, it is not suitable, gray the control...
        bSerialPort3 = FALSE;
        Com3->SetCheck(2);  
    }

    // Has COM4 already been selected?
    if (nActivePortSave == COM4)
    {
        bSerialPort4 = TRUE;
        Com4->SetCheck(1);  
    }
    // is COM4 suitable for CSP devices?
    else if (cspGetCommInfo(COM4) == SERIAL_RS232)
    {
        // Yes, it is suitable
        bSerialPort4 = TRUE;
        if (nActivePortSave == COM4)
            Com4->SetCheck(1);  
        else
            Com4->SetCheck(0);  
    }
    else
    {
        // No, it is not suitable, gray the control...
        bSerialPort4 = FALSE;
        Com4->SetCheck(2);  
    }
    
    return TRUE;  // return TRUE unless you set the focus to a control
                  // EXCEPTION: OCX Property Pages should return FALSE
}

/*
 ** void CCommDialog::OnOK() 
 *
 *  FILENAME: C:\Symbol\Csp32Proj\VcCspDemo\Src\CommDialog.cpp
 *
 *  PARAMETERS:     void
 *
 *  DESCRIPTION:    This function causes the member variable m_nActivePort
 *                  to be updated with the user's choice of comm port settings
 *                  before the dialog box closes.
 *
 *  RETURNS:        void
 *
 */

void CCommDialog::OnOK() 
{
    // save the currently selected comm port setting...
    SetDlgItemInt(IDC_COMM_SELECT, nActivePortSave, TRUE);

    CDialog::OnOK();
}

/*
 ** void CCommDialog::OnCancel() 
 *
 *  FILENAME: C:\Symbol\Csp32Proj\VcCspDemo\Src\CommDialog.cpp
 *
 *  PARAMETERS:     void
 *
 *  DESCRIPTION:    This function allows this dialog box to close without
 *                  changing the comm port setting.
 *
 *  RETURNS:        void
 *
 */

void CCommDialog::OnCancel() 
{
    // just return, don't change the comm setting...    
    CDialog::OnCancel();
}

/*
 ** void CCommDialog::OnCom1() 
 *
 *  FILENAME: C:\Symbol\Csp32Proj\VcCspDemo\Src\CommDialog.cpp
 *
 *  PARAMETERS:     void
 *
 *  DESCRIPTION:    This function inspects COM1 to see if it can be used for
 *                  the CSP Device communications port. If the port is an
 *                  available RS-232 port, it is selected as the potential
 *                  comm port.
 *
 *  RETURNS:        void
 *
 */

void CCommDialog::OnCom1() 
{
    // get pointers to all of the controls...
    CButton *Com1 = (CButton *) GetDlgItem(IDC_COM1);
    CButton *Com2 = (CButton *) GetDlgItem(IDC_COM2);
    CButton *Com3 = (CButton *) GetDlgItem(IDC_COM3);
    CButton *Com4 = (CButton *) GetDlgItem(IDC_COM4);

    // does this represent a change in ports?
    if (nActivePortSave != COM1)
    {
        // before we change the port setting, is it an RS-232 port?
        if (cspGetCommInfo(COM1) == SERIAL_RS232)
        {
            // Yes, change the setting...
            bSerialPort1 = TRUE;
            nActivePortSave = COM1;
            Com1->SetCheck(1);  

            // and update the status of the other controls...
            if (bSerialPort2)
                Com2->SetCheck(0);  
            else
                Com2->SetCheck(2);  

            if (bSerialPort3)
                Com3->SetCheck(0);  
            else
                Com3->SetCheck(2);  

            if (bSerialPort4)
                Com4->SetCheck(0);  
            else
                Com4->SetCheck(2);  
        }
        else
        {
            // No, don't allow this control to be selected...
            bSerialPort1 = FALSE;
            Com1->SetCheck(2);  
        }
    }   
    else
        // Already selected, keep it that way...
        Com1->SetCheck(1);  

}

/*
 ** void CCommDialog::OnCom2() 
 *
 *  FILENAME: C:\Symbol\Csp32Proj\VcCspDemo\Src\CommDialog.cpp
 *
 *  PARAMETERS:     void
 *
 *  DESCRIPTION:    This function inspects COM2 to see if it can be used for
 *                  the CSP Device communications port. If the port is an
 *                  available RS-232 port, it is selected as the potential
 *                  comm port.
 *
 *  RETURNS:        void
 *
 */

void CCommDialog::OnCom2() 
{
    // get pointers to all of the controls...
    CButton *Com1 = (CButton *) GetDlgItem(IDC_COM1);
    CButton *Com2 = (CButton *) GetDlgItem(IDC_COM2);
    CButton *Com3 = (CButton *) GetDlgItem(IDC_COM3);
    CButton *Com4 = (CButton *) GetDlgItem(IDC_COM4);

    // does this represent a change in ports?
    if (nActivePortSave != COM2)
    {
        // before we change the port setting, is it an RS-232 port?
        if (cspGetCommInfo(COM2) == SERIAL_RS232)
        {
            // Yes, change the setting...
            bSerialPort2 = TRUE;
            nActivePortSave = COM2;
            Com2->SetCheck(1);  

            if (bSerialPort1)
                Com1->SetCheck(0);  
            else
                Com1->SetCheck(2);  

            if (bSerialPort3)
                Com3->SetCheck(0);  
            else
                Com3->SetCheck(2);  

            if (bSerialPort4)
                Com4->SetCheck(0);  
            else
                Com4->SetCheck(2);  
        }
        else
        {
            // No, don't allow this control to be selected...
            bSerialPort2 = FALSE;
            Com2->SetCheck(2);  
        }
    }   
    else
        // Already selected, keep it that way...
        Com2->SetCheck(1);  
}

/*
 ** void CCommDialog::OnCom3() 
 *
 *  FILENAME: C:\Symbol\Csp32Proj\VcCspDemo\Src\CommDialog.cpp
 *
 *  PARAMETERS:     void
 *
 *  DESCRIPTION:    This function inspects COM3 to see if it can be used for
 *                  the CSP Device communications port. If the port is an
 *                  available RS-232 port, it is selected as the potential
 *                  comm port.
 *
 *  RETURNS:        void
 *
 */

void CCommDialog::OnCom3() 
{
    // get pointers to all of the controls...
    CButton *Com1 = (CButton *) GetDlgItem(IDC_COM1);
    CButton *Com2 = (CButton *) GetDlgItem(IDC_COM2);
    CButton *Com3 = (CButton *) GetDlgItem(IDC_COM3);
    CButton *Com4 = (CButton *) GetDlgItem(IDC_COM4);

    // does this represent a change in ports?
    if (nActivePortSave != COM3)
        {
        // before we change the port setting, is it an RS-232 port?
        if (cspGetCommInfo(COM3) == SERIAL_RS232)
        {
            // Yes, change the setting...
            bSerialPort3 = TRUE;
            nActivePortSave = COM3;
            Com3->SetCheck(1);  

            if (bSerialPort1)
                Com1->SetCheck(0);  
            else
                Com1->SetCheck(2);  

            if (bSerialPort2)
                Com2->SetCheck(0);  
            else
                Com2->SetCheck(2);  

            if (bSerialPort4)
                Com4->SetCheck(0);  
            else
                Com4->SetCheck(2);  
        }
        else
        {
            // No, don't allow this control to be selected...
            bSerialPort3 = FALSE;
            Com3->SetCheck(2);  
        }
    }   
    else
        // Already selected, keep it that way...
        Com3->SetCheck(1);  
}

/*
 ** void CCommDialog::OnCom4() 
 *
 *  FILENAME: C:\Symbol\Csp32Proj\VcCspDemo\Src\CommDialog.cpp
 *
 *  PARAMETERS:     void
 *
 *  DESCRIPTION:    This function inspects COM4 to see if it can be used for
 *                  the CSP Device communications port. If the port is an
 *                  available RS-232 port, it is selected as the potential
 *                  comm port.
 *
 *  RETURNS:        void
 *
 */

void CCommDialog::OnCom4() 
{
    // get pointers to all of the controls...
    CButton *Com1 = (CButton *) GetDlgItem(IDC_COM1);
    CButton *Com2 = (CButton *) GetDlgItem(IDC_COM2);
    CButton *Com3 = (CButton *) GetDlgItem(IDC_COM3);
    CButton *Com4 = (CButton *) GetDlgItem(IDC_COM4);

    // does this represent a change in ports?
    if (nActivePortSave != COM4)
        {
        // before we change the port setting, is it an RS-232 port?
        if (cspGetCommInfo(COM4) == SERIAL_RS232)
        {
            // Yes, change the setting...
            bSerialPort4 = TRUE;
            nActivePortSave = COM4;
            Com4->SetCheck(1);  

            if (bSerialPort1)
                Com1->SetCheck(0);  
            else
                Com1->SetCheck(2);  

            if (bSerialPort2)
                Com2->SetCheck(0);  
            else
                Com2->SetCheck(2);  

            if (bSerialPort3)
                Com3->SetCheck(0);  
            else
                Com3->SetCheck(2);  
        }
        else
        {
            // No, don't allow this control to be selected...
            bSerialPort4 = FALSE;
            Com4->SetCheck(2);  
        }
    }   
    else
        // Already selected, keep it that way...
        Com4->SetCheck(1);  
}


/*
 ** void CCommDialog::OnCommtest() 
 *
 *  FILENAME: C:\Symbol\Csp32Proj\VcCspDemo\Src\CommDialog.cpp
 *
 *  PARAMETERS:     void
 *
 *  DESCRIPTION:    This function launches the CommTest.exe program
 *                  in response to a user button press.
 *
 *  RETURNS:        void
 *
 */

void CCommDialog::OnCommtest() 
{
    // execute the CommTest program to assist the user in determining
    // why they can't use a specific port or to show what ports are
    // currently available to this program...
    WinExec("..\\Commtest\\Commtest", SW_SHOW);
    
}
