/*******************************************************************
$Header: C:/Symbol/Csp32Proj/Csp32/src/vcs/Csp32.cpv  1.0  05 Mar 1999  10:00:00 kFallon $
 *
 *  Copyright 1999 Peninsula Solutions, Inc., all rights reserved.
 *  Copyright 1999 Symbol Technologies, Inc., all rights reserved.
 *
 *          ***************************************
 *          * This file was generated by:         *
 *          *       Peninsula Solutions, Inc.     *
 *          *       1265 S. Semoran Blvd.         *
 *          *       Suite 1247                    *     
 *          *       Winter Park, FL 32792         *
 *          *       Phone: (407) 673-6544         *
 *          *       Fax:   (407) 673-6545         *
 *          * E-mail: info@PeninsulaSolutions.com *
 *          *                                     *
 *          * under a grant from:                 *
 *          *       Symbol Technologies, Inc.     *
 *          *       Holtsville, NY                *
 *          ***************************************
 *
 * The user is granted a license to use and modify this file for their own
 * purposes so long as the Copyright information remains intact.
 *
 *    MODULE NAME:   Csp32.cpp
 *
 *    DESCRIPTION:
 *                   This file provides the user API interface
 *                   functions for Symbol's Consumer Scanning 
 *                   Products as Dynamic Link Library. When 
 *                   compiled into a DLL, the user can access 
 *                   all of the functions available for the 
 *                   Symbol CyberPen.
 *
 *    AUTHOR:        Kim Fallon
 *
 *    DATE CREATED:  03/05/99
 *
 *    HISTORY:
 *
 *       03/05/99    Initial Development
 *
$Log: C:/Symbol/Csp32Proj/Csp32/src/vcs/Csp32.cpv $
 *******************************************************************/

#define WIN32_LEAN_AND_MEAN

#ifndef STRICT
    #define STRICT
#endif

#ifndef WIN32
    #define WIN32
#endif

#pragma hdrstop

/** include files **/
#include "afxwin.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "winver.h"                 // DLL Version information
#include "winbase.h"                // File I/O for debug
#include "wsc.h"                    // Include Serial Comm functions
#include "Csp32.h"                  // function proptotypes for DLL

#ifdef _DEBUG
    #include <time.h>               // time stamps for debug files
    
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif

/** local definitions **/
#define STX                         ((char)0x02)    // <stx> character
                                    
// Command byte values...
#define INTERROGATE                 ((char)0x01)
#define CLEAR_BAR_CODES             ((char)0x02)
#define DOWNLOAD_PARAMETERS         ((char)0x03)
#define RESERVED_CODE_4             ((char)0x04)
#define POWER_DOWN                  ((char)0x05)
#define RESERVED_CODE_6             ((char)0x06)
#define UPLOAD                      ((char)0x07)
#define UPLOAD_PARAMETERS           ((char)0x08)
#define SEND_SIGNATURE              ((char)0x0C)

// CSP device status value...
#define NO_ERROR_ENCOUNTERED        ((long)-6)

#define MAXTIME                     ((long)  18)    // 18 ticks, ~ 1 second
#define MAXSIZE                     ((long)4096)    // maximum size of retrieved barcodes
#define RX_QUE_SIZE                 ((long)1024)    // communications RX buffer setting
#define TX_QUE_SIZE                 ((long) 512)    // communications Tx buffer setting

                                    
/* default settings */
long    nCspActivePort              = -1;           // no comm port selected
long    nCspPenStatus               = -1;           // no current pen status
long    nCspProtocolVersion         = -1;           // no protocol version available
long    nCspSystemStatus            = -1;           // no system status available

long    nCspStoredBarcodes          =  0;           // number of stored barcode strings
long    nCspRetryCount              =  5;           // default number of interrogation retries

/** private data **/
#ifdef _DEBUG
    BOOL    bCspDebugMode           = FALSE;        // flag indicating debug mode is active
    HANDLE  hCspDebug;                              // handle to debug text file
    char    szCspDebug[80];                         // Debug text
    DWORD   nCspDebugLength;                        // number of characters in debug text
    char    *lpszCspDebug;                          // pointer to a debug string
    char    szDebugTime[10];                        // Time stamp string
#endif

long    nCspVolume;                                 // Pen volume setting
long    nCspBarcodeRedundancy;                      // Pen barcode redundancy setting
long    nCspContinuousScanning;                     // Pen continuous scan setting

char    aCspTlBits[8];                              // Pen TL Bits
char    aCspUploadCount[4];                         // Pen upload count
char    aCspSignature[8];                           // Pen signature
char    szCspUserId[9];                             // Pen User ID
char    szCspSwVersion[9];                          // Pen software version
char    szCspPenId[9];                              // Pen ID
char    szCspBarData[MAXSIZE];                      // uploaded barcode storage
char    aByteBuffer[MAXSIZE];                       // temp storage of CSP device response

/** private functions **/
static long cspGetc(void);
static char cspLrcCheck(char aLrcBytes[], long nMaxLength);
static void cspInitParms(void);
static long cspSendCommand (char *aCommand, long nMaxLength);

/*
 ** static long cspGetc(void)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     void
 *
 *  DESCRIPTION:    This function retrieves a character from the 
 *                  active serial port. If the serial port does not
 *                  have a character within the MAXTIME period, the
 *                  function returns with a COMMUNICATIONS_ERROR.
 *
 *  RETURNS:        character read (zero or positive value)
 *
 *                  COMMUNICATIONS_ERROR if no character was read
 */

static long cspGetc(void)
{
    long i;
    int  chRead;

    // make sure that a port has been selected...   
    if (nCspActivePort >= COM1)
        {
        // a port is selected, can we retrieve any data from it?
        for (i = 0; i < MAXTIME; i++)
            {
            // try to read a character...
            if ((chRead = SioGetc(nCspActivePort)) != WSC_NO_DATA)
                {
                #ifdef _DEBUG
                    if (bCspDebugMode)
                        {
                        nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                                    "R[0x%02X] ", 
                                                    chRead);
                        WriteFile(  hCspDebug,          // handle to debug text file
                                    szCspDebug,         // buffer of data to write to file
                                    nCspDebugLength,    // number of bytes to write
                                    &nCspDebugLength,   // number of bytes written
                                    0);                 // no overlapped structure present
                        }
                #endif
                
                return((long) chRead);
                }

            // let other tasks run while we wait for a character...
            Sleep(50);
            }
        }

    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "[device time out]\r\n");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif

   // if we make it this far, no character was read. So...  
   return( COMMUNICATIONS_ERROR );
}

/*
 ** static char cspLrcCheck(char aLrcBytes[], long nMaxLength)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     aLrcBytes[] contains the bytes used for computing the LRC
 *                  nMaxLength determines how many bytes of aLrcBytes[] are used
 *                  to perform LRC checking.
 *
 *  DESCRIPTION:    Computes the Longitudinal Redundancy Check on a
 *                  string of specified length.
 *
 *  RETURNS:        The calculated LRC is returned.
 *
 */
   
static char cspLrcCheck(char aLrcBytes[], long nMaxLength)
{
    long i;
    unsigned char chLrc = 0;        // initialize LRC value

    // compute the LRC for the entire string...
    for (i = 0; i < nMaxLength; i++)
        chLrc ^= aLrcBytes[i];

    return( chLrc );
}

/*
 ** static void cspInitParms(void)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     void
 *
 *  DESCRIPTION:    Invalidates all values used by this DLL. Used
 *                  to indicate that previously stored data is no
 *                  longer valid, possibly because of a new comm
 *                  port setting.
 *
 *  RETURNS:        void
 *
 */

static void cspInitParms(void)
{
    nCspActivePort              = -1;   // no comm port selected
    nCspPenStatus               = -1;   // no current pen status
    nCspProtocolVersion         = -1;   // no protocol version available
    nCspSystemStatus            = -1;   // no system status available

    nCspStoredBarcodes          =  0;   // number of stored barcode strings
}

/*
 ** static long cspSendCommand (char *aCommand, long nMaxLength)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     aCommand[] is an array of command bytes to send
 *                  to the CSP device.
 *                  nMaxLength specifies how many bytes in aCommand[]
 *                  are to be sent to the CSP device.
 *
 *  DESCRIPTION:    This function handles the basic overhead associated
 *                  with sending the CSP device a command. Examples of
 *                  commands for the CSP device are:
 *                      Interrogation
 *                      Read Barcodes
 *                      Clear Barcodes
 *                      Power Down
 *                      Get parameter
 *                      Set parameter
 *
 *                  aByteBuffer[0] contains the first data byte read
 *                  from the CSP device.
 *
 *  RETURNS:        STATUS_OK
 *                  COMMUNICATIONS_ERROR
 *                  INVALID_COMMAND_NUMBER
 *                  COMMAND_LRC_ERROR
 *                  RECEIVED_CHARACTER_ERROR
 *                  GENERAL_ERROR
 */

static long cspSendCommand (char *aCommand, long nMaxLength)
{
    long i;
    
    // make sure we have a valid port selected...
    if (nCspActivePort < COM1)
        return( COMMUNICATIONS_ERROR );

    // delay before sending message
    Sleep ( 120 );

    // clear the receive queue...
    while (SioGetc(nCspActivePort) >= 0);

    // send the command string...
    for ( i = 0; i < nMaxLength; i++)
        {
        SioPutc(nCspActivePort, aCommand[i]);

        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "W[0x%02X] ", 
                                            aCommand[i]);
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        }

    // wait for the reply to be processed, or MAXTIME...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n\r\n    Pen Status:       ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    nCspPenStatus  = cspGetc();
    aByteBuffer[0] = (char) nCspPenStatus;

    // convert pen status to CSP status values
    nCspPenStatus *= -1;

    // if response not available, reply with error
    if ( nCspPenStatus > 0 )
        return( COMMUNICATIONS_ERROR );

    if ( nCspPenStatus != NO_ERROR_ENCOUNTERED )
        return ( nCspPenStatus );
    else
        return ( STATUS_OK );
}


/*******************************************************************
 *              Communications                                     *
 *******************************************************************/

/*
 ** long DLL_IMPORT_EXPORT cspInit(long nComPort)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     nComPort specifies a communications port such as
 *                  COM1, Com2, etc.
 *
 *  DESCRIPTION:    This function tries to open the communications port
 *                  specified by nComPort. f it is successful, the
 *                  port is configured for CSP device compatibility.
 *
 *                  This provides a single call to initialize the CSP
 *                  commnications and all of the software structures 
 *                  for the CSP interface. The intialization consists of:
 *                      1. Initialize all memory structures, queues, and pointers
 *                      2. Establish serial communications with the specified port
 *
 *  RETURNS:        STATUS_OK if the specified nComPort is now active
 *                  BAD_PARAM if nComPort is invalid
 *                  COMMUNICATIONS_ERROR if nCommPort could not be activated
 */

NoMangle long DLL_IMPORT_EXPORT cspInit(long nComPort)
{
    // is the user requesting a valid port?
    if (nComPort >= COM1)
        {
        // close any previously opened ports...
        if (nCspActivePort >= COM1)
            cspRestore();               

        // now try to initialize the specified port...
        if (SioReset(nComPort, RX_QUE_SIZE, TX_QUE_SIZE) < 0)
            {
            #ifdef _DEBUG
                if (bCspDebugMode)
                    {
                    nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                                "\r\nComPort %d could not be initialized.\r\n", 
                                                nComPort+1);
                    WriteFile(  hCspDebug,          // handle to debug text file
                                szCspDebug,         // buffer of data to write to file
                                nCspDebugLength,    // number of bytes to write
                                &nCspDebugLength,   // number of bytes written
                                0);                 // no overlapped structure present
                    }
            #endif

            // indicate that no port is active...
            return(nCspActivePort = COMMUNICATIONS_ERROR);
            }
        else
            {
            #ifdef _DEBUG
                if (bCspDebugMode)
                    {
                    nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                                "\r\nComPort %d is active.\r\n", 
                                                nComPort+1);
                    WriteFile(  hCspDebug,          // handle to debug text file
                                szCspDebug,         // buffer of data to write to file
                                nCspDebugLength,    // number of bytes to write
                                &nCspDebugLength,   // number of bytes written
                                0);                 // no overlapped structure present
                    }
            #endif

            // set the current active communications port...
            nCspActivePort = nComPort;
            }

        // set the port parameters for CSP device compatibility...
        SioParms(nCspActivePort, OddParity, OneStopBit, WordLength8);
        SioBaud(nCspActivePort, Baud2400);
        SioDTR(nCspActivePort, SET_LINE);           
        SioRTS(nCspActivePort, SET_LINE);           
        return (STATUS_OK);
        }
        
    return (BAD_PARAM);
}

/*
 ** long DLL_IMPORT_EXPORT cspRestore()
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     void
 *
 *  DESCRIPTION:    This function attempts to close open connections
 *                  to the serial port specified by nCspActivePort.
 *
 *  RETURNS:        STATUS_OK if the port was closed 
 *
 *                  COMMUNICATIONS_ERROR if the connection was never
 *                  made or cannot be closed
 */ 

NoMangle long DLL_IMPORT_EXPORT cspRestore()
{ 
    long nRetStatus = COMMUNICATIONS_ERROR;
    
    // are we attempting to close a valid port?
    if (nCspActivePort >= COM1)
        {
        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\nClosing ComPort %d.\r\n", 
                                            nCspActivePort+1);
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif

        // close any previously opened ports...
        nRetStatus = SioDone(nCspActivePort);
        } 
 
    // initialize the dll interface...
    cspInitParms();

    // return status...
    if (nRetStatus < 0)
        return (COMMUNICATIONS_ERROR);
    else
        return (STATUS_OK);
}

/*******************************************************************
 *              Basic Functions                                    *
 *******************************************************************/

/*
 ** long DLL_IMPORT_EXPORT cspReadData()
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     void
 *
 *  DESCRIPTION:    This function reads the following information stored
 *                  in the CSP device:
 *                      All Barcodes
 *                      Pen ID
 *                      Signature
 *
 *                  The information is stored within the DLL. In order 
 *                  to retreive the information, the user must call the 
 *                  corresponding functions listed below:
 *                      cspGetBarcode()
 *                      cspGetPenId()
 *                      cspGetSignature()
 *
 *  RETURNS:        Number of stored barcode strings if no error, otherwise:
 *                  COMMUNICATIONS_ERROR
 *                  INVALID_COMMAND_NUMBER
 *                  COMMAND_LRC_ERROR
 *                  RECEIVED_CHARACTER_ERROR
 *                  GENERAL_ERROR
 */

NoMangle long DLL_IMPORT_EXPORT cspReadData()
{
    long nRetStatus;

    // read the data from the CSP device
    nRetStatus = cspReadRawData(NULL, DETERMINE_SIZE);

    // if response not available, reply with error
    if ( nRetStatus < 0 )
        return ( nRetStatus );

    return( nCspStoredBarcodes );
}


/*
 ** long DLL_IMPORT_EXPORT cspClearBarCodes()
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     Void
 *
 *  DESCRIPTION:    Clear all of the bar codes stored in the CSP device.
 *
 *  RETURNS:        STATUS_OK
 *                  COMMUNICATIONS_ERROR
 *                  INVALID_COMMAND_NUMBER
 *                  COMMAND_LRC_ERROR
 *                  RECEIVED_CHARACTER_ERROR
 *                  GENERAL_ERROR
 */

char aClearBarCodesCmd[] =
    {
    CLEAR_BAR_CODES,         // Clear Bar Code Command
    STX,                     // opcode
    0x00,                    // NULL terminate the message
    0x00                     // LRC
    };

NoMangle long DLL_IMPORT_EXPORT cspClearBarCodes()
{
    long i;
    long nRetStatus;
    
    // see if the CSP device is connected...
    if ((nRetStatus = cspInterrogate()) != STATUS_OK)
        return( nRetStatus );
        
    // send the command to the CSP device...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            _strtime( szDebugTime );        // Get time stamp string
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n\r\nClear Barcodes: [Time: %s]\r\n", 
                                        szDebugTime);
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    nRetStatus = cspSendCommand (aClearBarCodesCmd, sizeof(aClearBarCodesCmd));

    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // Response looks good, get the entire message...
    i = 1;

    // get the STX character...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    STX:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i++] = (char) cspGetc();

    // get the NULL character...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    NULL:             ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i++] = (char) cspGetc();

    // verify the LRC...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    LRC:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i] = cspLrcCheck(aByteBuffer, i);
    if ( aByteBuffer[i] != (char) cspGetc())
        return( COMMAND_LRC_ERROR );

    return( STATUS_OK );
}

/*
 ** long DLL_IMPORT_EXPORT cspPowerDown()
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  DESCRIPTION:    Direct the pen to end communication and enter
 *                  STOP mode until the next trigger action. This function
 *                  returns on or before the pen issues an audible signal
 *                  confirming the power down command.
 *
 *  RETURNS:        STATUS_OK
 *                  COMMUNICATIONS_ERROR
 *                  INVALID_COMMAND_NUMBER
 *                  COMMAND_LRC_ERROR
 *                  RECEIVED_CHARACTER_ERROR
 *                  GENERAL_ERROR
 */

char aPowerDownCmd[] =
   {
   POWER_DOWN,              // Power down Command
   STX,                     // opcode
   0x00,                    // NULL terminate the message
   0x07                     // LRC
   };

NoMangle long DLL_IMPORT_EXPORT cspPowerDown()
{
    long i;
    long nRetStatus;

    // send the command to the CSP device...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            _strtime( szDebugTime );        // Get time stamp string
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n\r\nPower Down: [Time: %s]\r\n", 
                                        szDebugTime);
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    nRetStatus = cspSendCommand (aPowerDownCmd, sizeof(aPowerDownCmd));

    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // Response looks good, get the entire message...
    i = 1;

    // get the STX character...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    STX:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i++] = (char) cspGetc();

    // get the NULL character...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    NULL:             ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i++] = (char) cspGetc();

    // verify the LRC...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    LRC:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i] = cspLrcCheck( aByteBuffer, i );
    if ( aByteBuffer[i] != (char) cspGetc())
        return( COMMAND_LRC_ERROR );

    return( STATUS_OK );
}

/*******************************************************************
 *              CSP Data Get                                       *
 *******************************************************************/

/*
 ** long DLL_IMPORT_EXPORT cspGetBarcode(char szBarData[], long nBarcodeNumber, long nMaxLength)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     szBarData[] is a character array the user has allocated 
 *                  to hold the barcode data. The string will be null terminated.
 *                  nBarcodeNumber indicates which barcode stored in
 *                  the szCspBcString[] will be returned.
 *                  nMaxLength is the length of the allocated space including the
 *                  null terminator. If nMaxLength is set to DETERMINE_SIZE, the
 *                  function will return the length of the barcode without
 *                  copying any data. 
 *
 *  DESCRIPTION:    This function copies the barcode data from szCspBcString[] 
 *                  to szBarData[] for up to nMaxLength characters. If the 
 *                  barcode is longer it will be truncated.
 *
 *  RETURNS:        > 0     length of the barcode including the NULL terminator
 *                  BAD_PARAM if the requested barcode does not exist
 */

NoMangle long DLL_IMPORT_EXPORT cspGetBarcode(char szBarData[], long nBarcodeNumber, long nMaxLength)
{
    long i;
    long nBarFound;
    long nBarLength;
    
    // make sure the requested barcode is valid...
    if (nBarcodeNumber < 0)
        return( BAD_PARAM );

    if (nBarcodeNumber >= nCspStoredBarcodes)
        return( BAD_PARAM );

    // OK, go find the barcode...
    i = 0;
    nBarFound = 0;
    while (nBarFound < nBarcodeNumber)
        {
        // Look for the null terminator separating the strings...
        while (szCspBarData[i++] != 0);
        
        // bump count to the next barcode...
        nBarFound++;
        }   

    // OK, we've got the barcode, so process it...
    nBarLength = strlen(&szCspBarData[i]) + 1;

    // did the user only request the string's length?
    if (nMaxLength > DETERMINE_SIZE)
        {
        // get the maximum number of characters to copy...
        nMaxLength = min(nBarLength,nMaxLength);

        // copy the barcode...
        memcpy(szBarData, &szCspBarData[i], nMaxLength);
        
        // and null terminate the string...
        szBarData[nMaxLength-1] = 0;
        }

    return(nBarLength);
}

/*
 ** long DLL_IMPORT_EXPORT cspGetPenId(char szPenId[9], long nMaxLength)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     szPenId[] holds the returned data.
 *                  nMaxLength is the length of the allocated space including
 *                  the null terminator.
 *
 *  DESCRIPTION:    This function retrieves the Pen ID string from the CSP
 *                  device data stored in the DLL. The string is null terminated.
 *
 *  RETURNS:        The length of the Pen ID string including the null terminator.
 *
 */

NoMangle long DLL_IMPORT_EXPORT cspGetPenId(char szPenId[9], long nMaxLength)
{
    // did the user only request the string's length?
    if (nMaxLength > DETERMINE_SIZE)
        {
        // get the maximum number of characters to copy...
        nMaxLength = min(sizeof(szCspPenId),nMaxLength);

        // copy the Pen ID...
        memcpy(szPenId, szCspPenId, nMaxLength);
        
        // and null terminate the string...
        szPenId[nMaxLength-1] = 0;
        }

    return( sizeof(szCspPenId) );
}

/*
 ** long DLL_IMPORT_EXPORT cspGetSignature(char aSignature[8], long nMaxLength)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     aSignature[] holds the returned data.
 *                  nMaxLength is the length of the allocated space including
 *                  the null terminator.
 *
 *  DESCRIPTION:    This function retrieves the Signature string from the CSP
 *                  device data stored in the DLL.
 *
 *  RETURNS:        The length of the Pen's Signature string.
 *
 */

NoMangle long DLL_IMPORT_EXPORT cspGetSignature(char aSignature[8], long nMaxLength)
{
    // did the user only request the string's length?
    if (nMaxLength > DETERMINE_SIZE)
        {
        // get the maximum number of characters to copy...
        nMaxLength = min(sizeof(aCspSignature),nMaxLength);

        // copy the Pen's Signature...
        memcpy(aSignature, aCspSignature, nMaxLength);
        }

    return( sizeof(aCspSignature) );
}

/*
 ** long DLL_IMPORT_EXPORT cspGetProtocol()
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     void
 *
 *  DESCRIPTION:    Returns an integer corresponding to the Protocol
 *                  byte from the CSP device.
 *
 *  RETURNS:        >= Current Protocol version information
 *                  < 0, Protocol version not available
 */

NoMangle long DLL_IMPORT_EXPORT cspGetProtocol()
{
    return (nCspProtocolVersion);
}

/*
 ** long DLL_IMPORT_EXPORT cspGetSystemStatus()
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     void
 *
 *  DESCRIPTION:    Returns an integer corresponding to the System 
 *                  Status byte from the CSP device.
 *
 *  RETURNS:        >= Current System Status information
 *                  < 0, System Status not available
 */

NoMangle long DLL_IMPORT_EXPORT cspGetSystemStatus()
{
    return (nCspSystemStatus);
}

/*
 ** long DLL_IMPORT_EXPORT cspGetSwVersion(char szSwVersion[9], long nMaxLength)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     szSwVersion[] holds the returned data.
 *                  nMaxLength is the length of the allocated space including
 *                  the null terminator.
 *
 *  DESCRIPTION:    This function retrieves the Software Version string from the CSP
 *                  device data stored in the DLL. The string is null terminated.
 *
 *  RETURNS:        The length of the Software Version string including the null terminator.
 *
 */

NoMangle long DLL_IMPORT_EXPORT cspGetSwVersion(char szSwVersion[9], long nMaxLength)
{
    // did the user only request the string's length?
    if (nMaxLength > DETERMINE_SIZE)
        {
        // get the maximum number of characters to copy...
        nMaxLength = min(sizeof(szCspSwVersion),nMaxLength);

        // copy the Software Version...
        memcpy(szSwVersion, szCspSwVersion, nMaxLength);
        
        // and null terminate the string...
        szSwVersion[nMaxLength-1] = 0;
        }

    return( sizeof(szCspSwVersion) );
}

/*******************************************************************
 *              CSP Configuration Set                              *
 *******************************************************************/

/*
 ** long DLL_IMPORT_EXPORT cspSetTlBits(char aTlBits[8], long nMaxLength)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     aTlBits[] specifies all of the bits which are 
 *                  to be stored.
 *                  nMaxLength is the length of the allocated space including
 *                  the null terminator.
 *
 *  DESCRIPTION:    This function saves 8 bytes (64 bits) of host 
 *                  specific data to the CSP device.
 *
 *  RETURNS:        STATUS_OK
 *                  COMMUNICATIONS_ERROR
 *                  INVALID_COMMAND_NUMBER
 *                  COMMAND_LRC_ERROR
 *                  RECEIVED_CHARACTER_ERROR
 *                  GENERAL_ERROR
 *                  BAD_PARAM if nMaxLength < 8
 */

NoMangle long DLL_IMPORT_EXPORT cspSetTlBits(char aTlBits[8], long nMaxLength)
{
    // did the user provide enough TL bits?
    if (nMaxLength < 8)
        return( BAD_PARAM );

    // set the TL Bits...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            lpszCspDebug = "TL Bits";
            }
    #endif
    return( cspSetParam( TL_BITS, aTlBits, nMaxLength ) );
}

/*
 ** long DLL_IMPORT_EXPORT cspSetVolume(long nVolume)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     Nvolume is one of the following values:
 *                      VOLUME_QUIET
 *                      VOLUME_LOW
 *                      VOLUME_MEDIUM
 *                      VOLUME_HIGH
 *
 *  DESCRIPTION:    This function sets the CSP device's beeper volume.
 *
 *  RETURNS:        STATUS_OK
 *                  COMMUNICATIONS_ERROR
 *                  INVALID_COMMAND_NUMBER
 *                  COMMAND_LRC_ERROR
 *                  RECEIVED_CHARACTER_ERROR
 *                  GENERAL_ERROR
 *                  BAD_PARAM if nVolume not in range of valid values.
 */

NoMangle long DLL_IMPORT_EXPORT cspSetVolume(long nVolume)
{
    char    aVolume[2];

    // is the volume setting valid?
    if ((nVolume < VOLUME_QUIET) || (nVolume > VOLUME_HIGH))
        return( BAD_PARAM );

    // set the Volume...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            lpszCspDebug = "Volume";
            }
    #endif
    aVolume[0] = (char) nVolume;
    return( cspSetParam( VOLUME, aVolume, 1 ) );
}

/*
 ** long DLL_IMPORT_EXPORT cspSetBarcodeRedundancy(long nOnOff)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     nOnOff is one of the following values:
 *                      PARAM_ON
 *                      PARAM_OFF
 *
 *  DESCRIPTION:    This function turns the barcode redundancy on the
 *                  CSP device on or off.
 *
 *  RETURNS:        STATUS_OK
 *                  COMMUNICATIONS_ERROR
 *                  INVALID_COMMAND_NUMBER
 *                  COMMAND_LRC_ERROR
 *                  RECEIVED_CHARACTER_ERROR
 *                  GENERAL_ERROR
 *                  BAD_PARAM if nOnOff not in range of valid values.
 */

NoMangle long DLL_IMPORT_EXPORT cspSetBarcodeRedundancy(long nOnOff)
{
    char    aBcRedundancy[2];

    // is the redundancy setting valid?
    if ((nOnOff < PARAM_OFF) || (nOnOff > PARAM_ON))
        return( BAD_PARAM );

    // set the Barcode Redundancy...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            lpszCspDebug = "Barcode Redundancy";
            }
    #endif
    aBcRedundancy[0] = (char) nOnOff;
    return( cspSetParam( BARCODE_REDUNDANCY, aBcRedundancy, 1 ) );
}

/*
 ** long DLL_IMPORT_EXPORT cspSetUserId(char szUserId[9], long nMaxLength)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     szUserId[] contains the string to be written to the 
 *                  CSP device.
 *                  nMaxLength is the length of the allocated space including
 *                  the null terminator.
 *
 *  DESCRIPTION:    This function writes the the User Id string to the CSP device.
 *
 *  RETURNS:        STATUS_OK
 *                  COMMUNICATIONS_ERROR
 *                  INVALID_COMMAND_NUMBER
 *                  COMMAND_LRC_ERROR
 *                  RECEIVED_CHARACTER_ERROR
 *                  GENERAL_ERROR
 *                  BAD_PARAM if nMaxLength is less than 8
 */

NoMangle long DLL_IMPORT_EXPORT cspSetUserId(char szUserId[9], long nMaxLength)
{
    // did the user provide enough data?
    if (nMaxLength < 8)
        return( BAD_PARAM );

    // set the User ID...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            lpszCspDebug = "User ID";
            }
    #endif
    return( cspSetParam( USER_ID, szUserId, 8 ) );
}

/*
 ** long DLL_IMPORT_EXPORT cspSetContinuousScanning(long nOnOff)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     nOnOff is one of the following values:
 *                      PARAM_ON
 *                      PARAM_OFF
 *
 *  DESCRIPTION:    This function turns the continuous scanning on the
 *                  CSP device on or off.
 *
 *  RETURNS:        STATUS_OK
 *                  COMMUNICATIONS_ERROR
 *                  INVALID_COMMAND_NUMBER
 *                  COMMAND_LRC_ERROR
 *                  RECEIVED_CHARACTER_ERROR
 *                  GENERAL_ERROR
 *                  BAD_PARAM if nOnOff not in range of valid values.
 */

NoMangle long DLL_IMPORT_EXPORT cspSetContinuousScanning(long nOnOff)
{
    char    aContScanning[2];

    // is the redundancy setting valid?
    if ((nOnOff < PARAM_OFF) || (nOnOff > PARAM_ON))
        return( BAD_PARAM );

    // set the Barcode Redundancy...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            lpszCspDebug = "Continuous Scanning";
            }
    #endif
    aContScanning[0] = (char) nOnOff;
    return( cspSetParam( CONTINUOUS_SCANNING, aContScanning, 1 ) );
}

/*******************************************************************
 *              CSP Configuration Get                              *
 *******************************************************************/

/*
 ** long DLL_IMPORT_EXPORT cspGetTlBits(char aTlBits[8], long nMaxLength)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:
 *                  nMaxLength is the length of the allocated space.
 *
 *  DESCRIPTION:    This function retrieves the TL Bits array from the 
 *                  CSP Device.
 *
 *  RETURNS:        if operation OK: number of bytes of TL Bits read
 *                  COMMUNICATIONS_ERROR
 *                  INVALID_COMMAND_NUMBER
 *                  COMMAND_LRC_ERROR
 *                  RECEIVED_CHARACTER_ERROR
 *                  GENERAL_ERROR
 */

NoMangle long DLL_IMPORT_EXPORT cspGetTlBits(char aTlBits[8], long nMaxLength)
{
    long nRetStatus;

    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            lpszCspDebug = "Get TL Bits";
            }
    #endif

    // get the TL Bits from the CSP device...
    nRetStatus = cspGetParam(TL_BITS, aCspTlBits, sizeof(aCspTlBits));

    // if response not available, reply with error
    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // did the user only request the string's length?
    if (nMaxLength > DETERMINE_SIZE)
        {
        // get the maximum number of characters to copy...
        nMaxLength = min(sizeof(aCspTlBits),nMaxLength);

        // copy the Pen's TL Bits...
        memcpy(aTlBits, aCspTlBits, nMaxLength);
        }

    return( sizeof(aCspTlBits) );
}

/*
 ** long DLL_IMPORT_EXPORT cspGetVolume()
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     void
 *
 *  DESCRIPTION:    This function retrieves the CSP device's volume setting.
 *
 *  RETURNS:        if the returned value is zero or greater:
 *                      VOLUME_QUIET
 *                      VOLUME_LOW
 *                      VOLUME_MEDIUM
 *                      VOLUME_HIGH
 *
 *                  if the returned value is negative:
 *                      COMMUNICATIONS_ERROR
 *                      INVALID_COMMAND_NUMBER
 *                      COMMAND_LRC_ERROR
 *                      RECEIVED_CHARACTER_ERROR
 *                      GENERAL_ERROR
 */

NoMangle long DLL_IMPORT_EXPORT cspGetVolume()
{
    long nRetStatus;
    char aVolume[2];

    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            lpszCspDebug = "Get Volume";
            }
    #endif

    // next, request the volume information...
    nRetStatus = cspGetParam(VOLUME, aVolume, 1);

    // if response not available, reply with error
    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // return the current volume setting
    nCspVolume = (long) aVolume[0];
    return( nCspVolume );
}

/*
 ** long DLL_IMPORT_EXPORT cspGetBarcodeRedundancy()
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     void
 *
 *  DESCRIPTION:    This function retieves the CSP device's barcode
 *                  redundancy setting.
 *
 *  RETURNS:        if the returned value is zero or greater:
 *                      PARAM_OFF
 *                      PARAM_ON
 *
 *                  if the returned value is negative:
 *                      COMMUNICATIONS_ERROR
 *                      INVALID_COMMAND_NUMBER
 *                      COMMAND_LRC_ERROR
 *                      RECEIVED_CHARACTER_ERROR
 *                      GENERAL_ERROR
 */

NoMangle long DLL_IMPORT_EXPORT cspGetBarcodeRedundancy()
{
    long nRetStatus;
    char aBcRedundancy[2];

    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            lpszCspDebug = "Get Barcode Redundancy";
            }
    #endif

    // request the barcode redundancy information...
    nRetStatus = cspGetParam(BARCODE_REDUNDANCY, aBcRedundancy, 1);

    // if response not available, reply with error
    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // return the current barcode redundancy setting
    nCspBarcodeRedundancy = (long) aBcRedundancy[0];
    return( nCspBarcodeRedundancy );
}

/*
 ** long DLL_IMPORT_EXPORT cspGetUserID(char szUserId[9], long nMaxLength)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     szUserId[] holds the returned data.
 *                  nMaxLength is the length of the allocated space including
 *                  the null terminator.
 *
 *  DESCRIPTION:    This function retrieves the User ID string from the CSP
 *                  device data stored in the DLL. The string is null terminated.
 *
 *  RETURNS:        The length of the User ID string including the null terminator.
 *                  COMMUNICATIONS_ERROR
 *                  INVALID_COMMAND_NUMBER
 *                  COMMAND_LRC_ERROR
 *                  RECEIVED_CHARACTER_ERROR
 *                  GENERAL_ERROR
 */

NoMangle long DLL_IMPORT_EXPORT cspGetUserID(char szUserId[9], long nMaxLength)
{
    long nRetStatus;

    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            lpszCspDebug = "Get User ID";
            }
    #endif

    // next, get the User ID from the CSP device...
    nRetStatus = cspGetParam(USER_ID, szCspUserId, sizeof(szCspUserId) - 1);

    // if response not available, reply with error
    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // did the user only request the string's length?
    if (nMaxLength > DETERMINE_SIZE)
        {
        // get the maximum number of characters to copy...
        nMaxLength = min(sizeof(szCspUserId),nMaxLength);

        // copy the User ID...
        memcpy(szUserId, szCspUserId, nMaxLength);
        
        // and null terminate the string...
        szUserId[nMaxLength-1] = 0;
        }

    return( sizeof(szCspUserId) );
}

/*
 ** long DLL_IMPORT_EXPORT cspGetContinuousScanning()
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     void
 *
 *  DESCRIPTION:    This function retieves the CSP device's continuous
 *                  scanning setting.
 *
 *  RETURNS:        if the returned value is zero or greater:
 *                      PARAM_OFF
 *                      PARAM_ON
 *
 *                  if the returned value is negative:
 *                      COMMUNICATIONS_ERROR
 *                      INVALID_COMMAND_NUMBER
 *                      COMMAND_LRC_ERROR
 *                      RECEIVED_CHARACTER_ERROR
 *                      GENERAL_ERROR
 */

NoMangle long DLL_IMPORT_EXPORT cspGetContinuousScanning()
{
    long nRetStatus;
    char aContScanning[2];

    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            lpszCspDebug = "Get Continuous Scanning";
            }
    #endif

    // next, request the continuous scanning information...
    nRetStatus = cspGetParam(CONTINUOUS_SCANNING, aContScanning, 1);

    // if response not available, reply with error
    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // return the current continuous scan setting
    nCspContinuousScanning = (long) aContScanning[0];
    return( nCspContinuousScanning );
}

/*******************************************************************
 *              DLL Configuration                                  *
 *******************************************************************/

/*
 ** long DLL_IMPORT_EXPORT cspSetRetryCount(long nRetryCount)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     nRetryCount specifies the number of interrogation retries.
 *                  The valid range for nRetryCount is [0..9]
 *
 *  DESCRIPTION:    This function sets the communications retry count. The 
 *                  default setting is 5. In the event of a communications
 *                  failure, the DLL will attempt the device interrogation
 *                  nRetryCount more times before returning a communications
 *                  error. This allows the user time to activate the CSP
 *                  device and connect it to the system in a reasonable time
 *                  period.
 *
 *  RETURNS:        STATUS_OK
 *                  BAD_PARAM if nRetryCount not in range of valid values.
 */

NoMangle long DLL_IMPORT_EXPORT cspSetRetryCount(long nRetryCount)
{
    // make sure the retry count is within the valid range...
    if ((nRetryCount < 0) || (nRetryCount > 9))
        return( BAD_PARAM );
        
    // change the retry count as indicated
    nCspRetryCount = nRetryCount;

    return( STATUS_OK );
}

/*
 ** long DLL_IMPORT_EXPORT cspGetRetryCount()
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     void
 *
 *  DESCRIPTION:    This function returns the current value of nRetryCount.
 *
 *  RETURNS:        The value of nRetryCount
 *
 */

NoMangle long DLL_IMPORT_EXPORT cspGetRetryCount()
{
    return (nCspRetryCount);
}

/*******************************************************************
 *              Miscellaneous                                      *
 *******************************************************************/

/*
 ** long DLL_IMPORT_EXPORT cspGetDllVersion(char szDllVersion[], long nMaxLength)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     szDllVersion[] holds the returned string.
 *                  nMaxLength is the length of the allocated space including
 *                  the null terminator.
 *
 *                  If this function is part of a DLL, it will return the 
 *                  FileVersion information about the DLL.
 *
 *                  If this function is statically linked with an application,
 *                  then the FileVersion information for the application is
 *                  returned.
 *
 *                  The correct operation of this function is tied to the 
 *                  Resource Information provided by VERSIONINFO in the 
 *                  FileVersion string. Modify the appropriate resource file
 *                  to change this setting.
 *
 *  RETURNS:        STATUS_OK           FileVersion info was read and returned
 *                  FILE_NOT_FOUND      FileVersion info could not be obtained
 */

NoMangle long DLL_IMPORT_EXPORT cspGetDllVersion(char szDllVersion[], long nMaxLength)
{
    char    fileName[_MAX_PATH];    // name of this executable code
    BYTE    block[2048];            // information block about this file
    DWORD   FAR *translation;       // pointer to the string translation(s)
    DWORD   FAR *buffer;            // a pointer into the FileVersion string
    DWORD   bytes;                  // number of bytes read
    DWORD   handle;                 // a handle to the file
    char    name[64];               // a name of the string to retreive

    //  Note!
    //
    //      If this DLL is dynamically linked against the MFC
    //      DLLs, any functions exported from this DLL which
    //      call into MFC must have the AFX_MANAGE_STATE macro
    //      added at the very beginning of the function.
    //
    //      For example:
    //
    //      extern "C" BOOL PASCAL EXPORT ExportedFunction()
    //      {
    //          AFX_MANAGE_STATE(AfxGetStaticModuleState());
    //          // normal function body here
    //      }
    //
    //      It is very important that this macro appear in each
    //      function, prior to any calls into MFC.  This means that
    //      it must appear as the first statement within the 
    //      function, even before any object variable declarations
    //      as their constructors may generate calls into the MFC
    //      DLL.
    //
    //      Please see MFC Technical Notes 33 and 58 for additional
    //      details.
    //
    AFX_MANAGE_STATE(AfxGetStaticModuleState());

    // Get the filename of THIS DLL!
    GetModuleFileName(AfxGetInstanceHandle(), fileName, _MAX_PATH);

    // Get the size of the information block for this dll...
    bytes = GetFileVersionInfoSize(fileName, &handle);

    // is there any information?
    if (bytes)
        {
        // make sure we don't read too much data...
        if (bytes > 2048)
            bytes = 2048;

        // Get the actual block for the version information into memory...
        if ( GetFileVersionInfo(fileName, handle, bytes, block) )
            {
            // Get the File Version for THIS DLL by name...
            if ( VerQueryValue(block, "\\VarFileInfo\\Translation", 
                              (VOID FAR * FAR *) &translation, 
                              (UINT FAR *) &bytes) )
                {
                // Now that we have the "block" to translate, ask for the FileVersion...
                // Format a request for the FileVersion string...
                wsprintf(name, "\\StringFileInfo\\%04x%04x\\FileVersion",
                         LOWORD(*translation), HIWORD(*translation));

                // Go get the FileVersion string from the block...
                if ( VerQueryValue(block, name, (VOID FAR * FAR *) &buffer, (UINT FAR *) &bytes) )
                    {
                    // return up to nMaxLength characters of the version information string...
                    #ifdef _DEBUG
                        if (nMaxLength >= (long) (bytes+6))
                            {
                            memcpy( (LPSTR) szDllVersion, (LPSTR) "Debug ", nMaxLength);
                            strncat( (LPSTR) szDllVersion, (LPSTR) buffer, nMaxLength-6);
                            }
                        else
                            return(bytes+6);    // String would not fit!
                    #else
                        if (nMaxLength >= (long) bytes)
                            memcpy( (LPSTR) szDllVersion, (LPSTR) buffer, nMaxLength);
                        else
                            return(bytes);      // String would not fit!
                    #endif

                    // Say that everything was OK with the version information...
                    return(STATUS_OK);
                    }
                }

            }

        }

    // We had a problem somewhere, send back a COMMUNICATIONS_ERROR for status...
    return(FILE_NOT_FOUND);
}

/*******************************************************************
 *              Debug                                              *
 *******************************************************************/

/*
 ** long DLL_IMPORT_EXPORT cspSetDebugMode(long nOnOff)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     nOnOff is one of the following values:
 *                      PARAM_ON
 *                      PARAM_OFF
 *
 *  DESCRIPTION:    This function turns the debug mode of the DLL on or off.
 *                  
 *                  When the debug mode is on, all commands and reponses are written 
 *                  to a debug file named "debug.txt". Each entry in the file
 *                  will have a time stamp preceeding it.
 *
 *                  Note: this function function only writes data to the 
 *                  output file in the debug version of the DLL.
 *
 *  RETURNS:        STATUS_OK
 *                  FILE_NOT_FOUND
 *                  other: see WinError.h
 */

NoMangle long DLL_IMPORT_EXPORT cspSetDebugMode(long nOnOff)
{
    #ifdef _DEBUG
    // are we turning off the debug file?
    if (nOnOff == PARAM_OFF)
        {
        // did we open a file previously?
        if (bCspDebugMode)
            {
            // flag debug mode as terminated...
            bCspDebugMode = FALSE;

            // try to close the file...
            if ( !CloseHandle( hCspDebug ) )
                return( (long) GetLastError() );
            }
        else
            return( FILE_NOT_FOUND );
        }
    else
        {
        // create the debug file...
        hCspDebug = CreateFile("Debug.txt",                 // debug file name
                                GENERIC_WRITE,              // we are going to write this file
                                0,                          // file sharing disabled
                                0,                          // use default security descriptor
                                CREATE_ALWAYS,              // re-write previously existing file
                                FILE_ATTRIBUTE_NORMAL,      // default file attributes
                                0);                         // no template file provided

        // did we create the file successfully?
        if ( !hCspDebug )
            {
            // return the reason for the file creation error...
            return( (long) GetLastError() );            
            }

        // flag debug mode as active...
        bCspDebugMode = TRUE;

        nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                    "Consumer Scanning Products Debug File:");
        WriteFile(  hCspDebug,          // handle to debug text file
                    szCspDebug,         // buffer of data to write to file
                    nCspDebugLength,    // number of bytes to write
                    &nCspDebugLength,   // number of bytes written
                    0);                 // no overlapped structure present
        }
    #endif

    // everything is OK
    return (STATUS_OK);
}

/*
 ** long DLL_IMPORT_EXPORT cspGetCommInfo(long nComPort)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     nCommPort is in the range [COM1..COM16]
 *
 *  DESCRIPTION:    This function determines information about the 
 *                  specified comm port. 
 *
 *                  This function determines if the port specified
 *                  is a valid RS-232 serial port. It can detect
 *                  the following information about the serial port:
 *                      1. If it is a serial port (default RS-232)
 *                      2. If it is a modem (internal/external)
 *                      3. If the port is in use by another program
 *                      4. If the port does not exist
 *                      5. If there is some other Windows related error
 *                         associated with the port.
 *
 *  RETURNS:        SERIAL_MODEM -  the port is a modem or connected to 
 *                                  an external modem
 *                  SERIAL_RS232  - the port is an RS-232 port
 *                  ACCESS_DENIED - the port is in use by another program
 *                  FILE_NOT_FOUND- the port does not exist
 *                  other         - consult WinError.h
 */

NoMangle long DLL_IMPORT_EXPORT cspGetCommInfo(long nComPort)
{
    DWORD TimeMark;     // used for a measured time delay
    long  i;                // data read from modem/serial port
    long  portType;     // SERIAL_RS232 | SERIAL_MODEM if port is valid
    long  Next;         // index into an array
    char *Expect;       // pointer to expected modem results

    portType = SERIAL_RS232;    // Assume that this is a valid RS-232 port
    Expect = "OK";              // expect OK if we connect to a modem
    Next = 0;                   // index into Expect[] 

    // see if we can access the port...
    if ( SioReset(nComPort, RX_QUE_SIZE, TX_QUE_SIZE) < 0)
        return( SioWinError(NULL, 0));  // not accessible, report the problem

    // set the DTR and RTS control lines
    SioDTR(nComPort,'S');
    SioRTS(nComPort,'S');

    // send out a carriage return to intialize the port...
    SioPutc(nComPort,'\r');
    
    // if this is a modem, it should set DSR at this point...
    if (SioDSR(nComPort) != 0)      // modem must set DSR
      {
        // A wrap around connector will set DSR too, so we have to verify the "modem"
        // send AT command to the "modem"
        SioPutc(nComPort,'\r');
        Sleep(250);                 // let other tasks run 
        SioPutc(nComPort,'A');
        Sleep(250);                 // let other tasks run 
        SioPutc(nComPort,'T');
        Sleep(250);                 // let other tasks run 
        SioPutc(nComPort,'\r');

        // expect OK back within 2 seconds if this is a modem
        Next = 0;
        TimeMark = SioTimer() + 2000;
        while (SioTimer() < TimeMark)
          {
           // read any incoming data from the serial port...
           i = SioGetc(nComPort);
           if (i > -1)
             {
              // OK, we got a character. Now compare it to the "Expected" response...
              if ((char)i == Expect[Next])
                {
                 Next++;
                 if (Next == 2)
                   {       
                    portType = SERIAL_MODEM;
                    break;
                   }
                }
             }
          } // end while
      }

    // Release the port, we have our information...
    SioDone(nComPort);

    // tell the user what kind of device we have...
    return (portType);
}


/*******************************************************************
 *              Advanced Functions                                 *
 *******************************************************************/

/*
 ** long DLL_IMPORT_EXPORT cspReadRawData(char aBuffer[], long nMaxLength )
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     aBuffer[] array where data will be copied after it
 *                  is read out of the pen.
 *                  nMaxLength is the length of the allocated array.
 *                  If nMaxLength is set to DETERMINE_SIZE, the function will 
 *                  return the length of the barcode without copying any data.
 *
 *  DESCRIPTION:    This function performs an "Upload" command on a CSP device. 
 *                  The raw data read from the CSP device is placed into 
 *                  aBuffer[] for up to nMaxLength bytes.  If the amount of 
 *                  data that is read from the CSP device exceeds nMaxLength, 
 *                  the buffer will be truncated at nMaxLength bytes. 
 *
 *                  This function will retry the interrogation upto
 *                  nCspRetryCount times before aborting the operation.
 *
 *                  Protocol Change effective 4/1/98: Signature field may
 *                  or may not be present: it can be disabled. This code now
 *                  takes that in to account.
 *
 *                  This function asks the pen if the signature is enabled
 *                  before it starts an upload. So long as the comunications
 *                  works, it performs the upload.
 *
 *                  If the signature is enabled, and if the last counted 
 *                  string is 8 characters long, it is assumed to be the 
 *                  signature string.  
 *
 *                  If it is not eight characters long, or the signature 
 *                  is disabled, then the Signature field is returned as 
 *                  'Disabled'.
 *
 *  RETURNS:        Number of bytes read if no error, otherwise:
 *                  COMMUNICATIONS_ERROR
 *                  INVALID_COMMAND_NUMBER
 *                  COMMAND_LRC_ERROR
 *                  RECEIVED_CHARACTER_ERROR
 *                  GENERAL_ERROR
 */

char aUploadCmd[] =
    {
    UPLOAD,                  // Upload Command
    STX,                     // opcode
    0x00,                    // NULL terminate the message
    0x05                     // LRC
    };

NoMangle long DLL_IMPORT_EXPORT cspReadRawData(char aBuffer[], long nMaxLength )
{
    long i, k, n, iSigStart;
    long nRetStatus;
    char j;
    char aSignature[2];

    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            lpszCspDebug = "Signature";
            }
    #endif

    // find out if the signature field is enabled...
    nRetStatus = cspGetParam(SEND_SIGNATURE, aSignature, 1);

    // if response not available, reply with error
    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // send the command to the CSP device...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            _strtime( szDebugTime );        // Get time stamp string
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n\r\nRead Data: [Time: %s]\r\n", 
                                        szDebugTime);
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    nRetStatus = cspSendCommand (aUploadCmd, sizeof(aUploadCmd));

    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // Response looks good, get the entire message...
    i = 1;

    // get the STX character...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    STX:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i++] = (char) cspGetc();

    // get the pen id characters...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    Pen ID:           ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    for ( j = 0; j < 8 ; j++ )
        aByteBuffer[i++] = szCspPenId[j] = (char) cspGetc();

    // get the upload count characters...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    Upload Count:     ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    for ( j = 0; j < 4 ; j++ )
        aByteBuffer[i++] = aCspUploadCount[j] = (char) cspGetc();

    // read in the counted strings until a NULL occurs...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    Counted String:   ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    nCspStoredBarcodes = iSigStart = k = n = 0;
    while (( aByteBuffer[i++] = j = (char) cspGetc()) > 0x00 )
        {
        // get the bar code string characters...
        iSigStart = k;
        for ( n = 0; n < (long) j; n++ )
            aByteBuffer[i++] = szCspBarData[k++] = (char) cspGetc();
        // insert a <NULL> between strings
        szCspBarData[k++] = 0;
        nCspStoredBarcodes++;

        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n                      ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        }

    // "n" has the length of the last counted string...
    // if "n" is exactly 8 characters, 
    // and the signature is enabled, 
    // then the last 8 characters are the signature...
    if (( n == 8 ) && (aSignature[0] == PARAM_ON))
        {
        // get the signature characters from the last counted string...
        for ( k = 0; k < sizeof(aCspSignature); k++ )
            aCspSignature[k] = szCspBarData[iSigStart+k];

        // the signature string is not counted as a barcode!
        nCspStoredBarcodes--;
        }
    else 
        {
        // Label the signature field 'Disabled'...
        memcpy(aCspSignature, "Disabled", sizeof(aCspSignature));
        }

    // verify the LRC...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    LRC:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i] = cspLrcCheck(aByteBuffer, i); 
    if ( aByteBuffer[i] != (char) cspGetc())
        {
        return( COMMAND_LRC_ERROR );
        }

    // should we copy the data to the user's array?
    if (aBuffer)
        {
        if (nMaxLength)
            memcpy(aBuffer, aByteBuffer, nMaxLength);
        }

    return(i + 1);
}

/*
 ** long DLL_IMPORT_EXPORT  cspSetParam( long nParam, char *szString, long nMaxLength )
 *
 *  FILENAME: D:\ipSDK\Src\ipsdk32.c
 *
 *  PARAMETERS:      param_number is the CSP device parameter to change
 *                   *string points to the new parameter setting
 *                   length specifies how many characters in string[] make
 *                   up the new parameter setting.
 *
 *  DESCRIPTION:     The cspSetParam function permits the user to write 
 *                   individual pen parameters one at a time rather than 
 *                   batching all of the parameters for one download.
 *
 *  RETURNS:
 *                   STATUS_OK
 *                   COMMUNICATIONS_ERROR
 *                   INVALID_COMMAND_NUMBER
 *                   COMMAND_LRC_ERROR
 *                   RECEIVED_CHARACTER_ERROR
 *                   GENERAL_ERROR
 */

NoMangle long DLL_IMPORT_EXPORT cspSetParam( long nParam, char szString[], long nMaxLength )
{
    long nRetStatus;
    long i = 0;
    long k;
    char j;

    // see if the CSP device is connected...
    if ((nRetStatus = cspInterrogate()) != STATUS_OK)
        return( nRetStatus );
        
    // start the download packet...
    aByteBuffer[i++] = DOWNLOAD_PARAMETERS;
    aByteBuffer[i++] = STX;

    // set the length of the counted string...
    aByteBuffer[i++] = nMaxLength + 1;

    // specify the parameter number...
    aByteBuffer[i++] = (char) nParam;

    // copy the string into the buffer...
    for (k = 0; k < nMaxLength; k++)
        aByteBuffer[i++] = szString[k];

    // null terminate all strings, assumed
    aByteBuffer[i++] = 0x00;    

    // perform LRC Check on the string...
    aByteBuffer[i] = cspLrcCheck( aByteBuffer, i );
    i++;

    // send the command to the CSP device...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            _strtime( szDebugTime );        // Get time stamp string
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n\r\nSet Parameter: %s [Time: %s]\r\n", 
                                        lpszCspDebug, 
                                        szDebugTime);
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    nRetStatus = cspSendCommand( aByteBuffer, i );

    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // Response looks good, get the entire message...
    i = 1;

    // get the STX character...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    STX:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i++] = (char) cspGetc();

    // read in the counted string until a NULL occurs...
    // first byte "j" is the length of the counted string...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    Counted String:   ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    while (( aByteBuffer[i++] = j = (char) cspGetc()) > 0x00 )
        {
        // get the parameter character...
        aByteBuffer[i++] = (char) cspGetc();

        // read the parameter value into aByteBuffer[] upto j-1 characters...
        for (k = 0; k < (long) (j-1); k++)
            {
            aByteBuffer[i++] = (char) cspGetc();
            }

        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n                      ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        }

    // verify the LRC...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    LRC:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i] = cspLrcCheck(aByteBuffer, i);
    if ( aByteBuffer[i] != (char) cspGetc())
        return( COMMAND_LRC_ERROR );

    return( STATUS_OK );
}
   
/*
 ** long DLL_IMPORT_EXPORT  cspGetParam( long nParam, char *szString, long nMaxLength )
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:      nParam is the CSP device parameter to retrieve
 *                   *szString stores the retrieved parameter value
 *                   nMaxLength specifies the maximum number of characters that
 *                   can be stored in *string.
 *
 *  DESCRIPTION:     The cspGetParam function permits the user to read
 *                   individual pen parameters one at a time.
 *
 *  RETURNS:         STATUS_OK
 *                   COMMUNICATIONS_ERROR
 *                   INVALID_COMMAND_NUMBER
 *                   COMMAND_LRC_ERROR
 *                   RECEIVED_CHARACTER_ERROR
 *                   GENERAL_ERROR
 */

char aGetParametersCmd[] =
    {
    UPLOAD_PARAMETERS,       
    STX,                     

    0x01,                    // Read one parameter
    0x00,                    // TBD parameter

    0x00,                    // NULL terminate the message
    0x00                     // LRC
    };

#define GPC_PARM_START  ((long) 3)
#define GPC_SIZE        (sizeof(aGetParametersCmd) - 1)

NoMangle long DLL_IMPORT_EXPORT cspGetParam( long nParam, char szString[], long nMaxLength )
{
    long i, k;
    long nRetStatus;
    char j;

    // see if the CSP device is connected...
    if ((nRetStatus = cspInterrogate()) != STATUS_OK)
        return( nRetStatus );
        
    // fill in the command string...
    aGetParametersCmd[GPC_PARM_START] = (char) nParam;
    aGetParametersCmd[GPC_SIZE] = cspLrcCheck(aGetParametersCmd, GPC_SIZE);

    // send the command to the CSP device...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            _strtime( szDebugTime );        // Get time stamp string
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n\r\nGet Parameter: %s [Time: %s]\r\n", 
                                        lpszCspDebug, 
                                        szDebugTime);
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    nRetStatus = cspSendCommand (aGetParametersCmd, sizeof(aGetParametersCmd));

    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // Response looks good, get the entire message...
    i = 1;

    // get the STX character...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    STX:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i++] = (char) cspGetc();

    // read in the counted string until a NULL occurs...
    // first byte "j" is the length of the counted string...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    Counted String:   ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    while (( aByteBuffer[i++] = j = (char) cspGetc()) > 0x00 )
        {
        // get the parameter character...
        aByteBuffer[i++] = (char) cspGetc();

        // read the parameter value into szString[] upto nMaxLength characters...
        for (k = 0; k < (long) (j-1); k++)
            {
            if (k < nMaxLength)
                aByteBuffer[i++] = szString[k] = (char) cspGetc();
            // parameter exceeded szString[], don't copy over buffer...
            else
                aByteBuffer[i++] = (char) cspGetc();
            }

        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n                      ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        }

    // verify the LRC...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    LRC:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i] = cspLrcCheck(aByteBuffer, i);
    if ( aByteBuffer[i] != (char) cspGetc())
        return( COMMAND_LRC_ERROR );

    return( STATUS_OK );
}



/*
 ** long DLL_IMPORT_EXPORT  cspInterrogate(void)
 *
 *  FILENAME: C:\Symbol\Csp32Proj\Csp32\Src\Csp32.cpp
 *
 *  PARAMETERS:     void
 *
 *  DESCRIPTION:    Request a response from the CSP device indicating that it is 
 *                  operating and containing version information. Valid response 
 *                  information is saved in local storage.
 *
 *                  The interrograte function will automatically retry the 
 *                  interrogation operation up to nCspRetryCount times before 
 *                  aborting.
 *
 *  RETURNS:        STATUS_OK
 *                  COMMUNICATIONS_ERROR
 *                  INVALID_COMMAND_NUMBER
 *                  COMMAND_LRC_ERROR
 *                  RECEIVED_CHARACTER_ERROR
 *                  GENERAL_ERROR
 */

char aInterrogateCmd[] =
    {
    INTERROGATE,      // Interrogate Command
    STX,              // opcode
    0x00,             // NULL terminate the message
    0x03              // LRC
    };

NoMangle long DLL_IMPORT_EXPORT cspInterrogate(void)
{
    long i, j;
    long nRetStatus;
    long nCount = nCspRetryCount;

    // interrogate the CSP device up to nCspRetryCount+1 times...
    do
        {
        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                _strtime( szDebugTime );        // Get time stamp string
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n\r\nInterrogate: [Time: %s]\r\n", 
                                            szDebugTime);
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif

        // invalidate the old interrogation status...
        nCspPenStatus               = -1;   // no current pen status
        nCspProtocolVersion         = -1;   // no protocol version available
        nCspSystemStatus            = -1;   // no system status available
        szCspUserId[0]              =  0;   // no user Id available
        szCspSwVersion[0]           =  0;   // no software version available
        
        // make sure that a comm port has been selected...
        if (nCspActivePort < COM1)
            {
            #ifdef _DEBUG
                if (bCspDebugMode)
                    {
                    nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                                "\r\nNo valid COM port selected; Interrogation aborted.\r\n");
                    WriteFile(  hCspDebug,          // handle to debug text file
                                szCspDebug,         // buffer of data to write to file
                                nCspDebugLength,    // number of bytes to write
                                &nCspDebugLength,   // number of bytes written
                                0);                 // no overlapped structure present
                    }
            #endif

            return( COMMUNICATIONS_ERROR );
            }

        // send the command to the CSP device...
        nRetStatus = cspSendCommand (aInterrogateCmd, sizeof(aInterrogateCmd));

        // if the pen reported an error, try again from the top...
        if (nRetStatus != STATUS_OK)
            {
            continue;
            }

        // Response looks good, get the entire message...
        i = 1;

        // get the STX character...
        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n    STX:              ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        aByteBuffer[i++] = (char) cspGetc();

        // get the protocol version character...
        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n    Protocol Version: ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        nCspProtocolVersion = cspGetc();
        aByteBuffer[i++] = (char) nCspProtocolVersion;

        // get the system status character...
        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n    System Status:    ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        nCspSystemStatus = cspGetc();
        aByteBuffer[i++] = (char) nCspSystemStatus;

        // get the user id characters...
        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n    User ID:          ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        for ( j = 0; j < 8 ; j++ )
            aByteBuffer[i++] = szCspUserId[j] = (char) cspGetc();

        // get the software version characters...
        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n    Software Version: ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        for ( j = 0; j < 8 ; j++ )
            aByteBuffer[i++] = szCspSwVersion[j] = (char) cspGetc();

        // get the NULL character...
        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n    NULL:             ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        aByteBuffer[i++] = (char) cspGetc();

        // verify the LRC...
        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n    LRC:              ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        aByteBuffer[i] = cspLrcCheck( aByteBuffer, i );
        if ( aByteBuffer[i] != ( unsigned char ) cspGetc())
            {
            nRetStatus = COMMAND_LRC_ERROR;
            continue;
            }

        // CSP device was interrogated successfully...
        return( STATUS_OK );

        } while (nCount--);

    return (nRetStatus);            // interrogation failed
}

/*******************************************************************
 *              Th, Th, Th, Th, That's All Folks !                 *
 *******************************************************************/
